"""
Toxopy (https://github.com/bchaselab/Toxopy)
© M. Alyetama, University of Nebraska at Omaha
Licensed under the terms of the MIT license
"""

import pandas as pd
from scipy.stats import ks_2samp


def turnpoints_time_diff(csv_file, cat):

    # csv_file is typivally "all_turnpoints_with_owner_improved.csv"
    # use the outpt of this function in ks_2samp(output1, output2)

    df = pd.read_csv(csv_file)

    df = df.loc[(df['cat'] == cat)]

    df = df.reset_index()

    def altElement(a):

        firstispeak = str(df['firstispeak'][0])

        if firstispeak == 'False':
            return a[::2]

        elif firstispeak == 'True':
            return a[1::2]

    ls = df[df['cat'] == cat]['time'].tolist()

    n = len(altElement(ls))

    time_diff = []

    for i in range(0, n):
        if i == n-1:
            break
        else:
            time_diff.append(altElement(ls)[i+1] - altElement(ls)[i])

    return [round(x, 4) for x in time_diff]



"""
Toxopy (https://github.com/bchaselab/Toxopy)
© M. Alyetama, University of Nebraska at Omaha
Licensed under the terms of the MIT license
"""

import pandas as pd
from pathlib import Path


def improve_turnpoints(csv_file, trial_type, output_dir):

    df = pd.read_csv(csv_file)

    tls, times, proba = [], [], []

    for x in df['tppos']:
        times.append(round(x / 30, 4))
    for y in df['proba']:
        proba.append(format(y, '.8f'))

    df = df.drop(columns=['proba'])

    k, j, t = 300, 180, 120

    if trial_type == "with_owner":
        for i in times:
            if i < k:
                tls.append('FT')
            elif k < i < k + j:
                tls.append('ST1')
            elif k + j < i < k + j * 2:
                tls.append('UT1')
            elif k + j * 2 < i < k + j * 3:
                tls.append('ST2')
            elif k + j * 3 < i <= k + j * 4:
                tls.append('UT2')

    elif trial_type == "cat_alone":
        for i in times:
            if i < t:
                tls.append('CA1')
            elif t < i < t * 2:
                tls.append('CA2')
            elif t * 2 < i < t * 3:
                tls.append('CA3')
            elif t * 3 < i < t * 4:
                tls.append('CA4')
            elif t * 4 < i <= t * 5:
                tls.append('CA5')

    df['proba'], df['trial'], df['time'] = proba, tls, times

    df.to_csv(f'{output_dir}/{Path(csv_file).stem}_improved.csv',
              index=False,
              sep=',',
              encoding='utf-8')



import pandas as pd
from pathlib import Path
import glob
from toxopy import trials_cap, trials
from os import remove
from tqdm import tqdm


def correct_times(csv_dir, output_dir):

	"""'csv_dir' is the path to the folder containing the combined csv files"""

	files = glob.glob(f'{csv_dir}/*.csv')

	tls = trials_cap()

	trials_times = [300, 420, 600, 720, 900, 1020, 1200, 1320, 1500, 1620]

	additions = [0, 300, 120, 480, 240, 660, 360, 840, 480, 1020]


	def subdf(variable):

		return df[df['trial'] == trial][variable]


	for file in tqdm(files):

		df = pd.read_csv(file, header=[0])

		cat = Path(file).stem

		for trial, ttime, ad in zip(tls, trials_times, additions):

			time = df.loc[(df['trial'] == trial)]['time']

			diff = max(time) - min(time)

			if trial == trial and diff > ttime:

				raise ValueError('Fails!', cat, trial, diff)

			elif trial == trial and diff < ttime:

				t, v, a, r = subdf('time') + ad, subdf('velocity_loess05'), subdf('acceleration_loess05'), subdf('trial')

				fdf = pd.DataFrame([t, v, a, r]).T

				fdf.to_csv(f'{output_dir}/{cat}_{trial}.csv', index=False, encoding='utf-8-sig')

		fs = glob.glob(f'{output_dir}/{cat}_*.csv')

		ccsv = pd.concat([pd.read_csv(i) for i in fs])

		old_names, new_names = trials_cap(), trials()

		ccsv['trial'] = ccsv['trial'].replace(old_names, new_names)

		ccsv = ccsv.sort_values(by=['time'])

		ccsv.to_csv(f'{output_dir}/{cat}.csv', index=False, encoding='utf-8-sig')

		[remove(i) for i in fs]



import pandas as pd
from glob import glob
from pathlib import Path


def one_cat_one_file(dir1, dir2, output_dir):

	"""'dir1' is the dir with cat_alone_improved csv files
	'dir2' is the dir with with_owner_improved csv files"""

	cat_alone, with_owner = glob(dir1 + '/*.csv'), glob(dir2 + '/*.csv')

	for f1, f2 in zip(sorted(cat_alone), sorted(with_owner)):

		cat1, cat2 = Path(f1).stem[:-10], Path(f2).stem[:-11]

		files = []

		if cat1 == cat2:
			files.append(f1)
			files.append(f2)

		else:
			raise ValueError('Something is wrong!')

		combined_csv = pd.concat([pd.read_csv(f, usecols = ['time', 'velocity_loess05', 'acceleration_loess05', 'trial']) for f in files])

		combined_csv.to_csv(f'{output_dir}/{cat1}.csv', index=False, encoding='utf-8-sig')



import pandas as pd
import numpy as np


def turning_points(array):
    ''' turning_points(array) -> min_indices, max_indices
    Finds the turning points within an 1D array and returns the indices of the minimum and 
    maximum turning points in two separate lists.
    Adapted from httpts://stackoverflow.com/a/48360671
    '''
    idx_max, idx_min = [[]] * 2
    if (len(array) < 3):
        return idx_min, idx_max

    STATIONARY, MAXIMA, MINIMA = range(3)

    def get_state(i, j):
        if i < j:
            return MAXIMA
        if i > j:
            return MINIMA
        return STATIONARY

    pts = get_state(array[0], array[1])
    start = 1
    for i in range(2, len(array)):
        x = get_state(array[i - 1], array[i])
        if x != STATIONARY:
            if pts != STATIONARY and pts != x:
                if x == MINIMA:
                    idx_max.append((start + i - 1) // 2)
                else:
                    idx_min.append((start + i - 1) // 2)
            start = i
            pts = x
    return idx_min, idx_max


def turning_points_output(csv_file, variable):

    df = pd.read_csv(csv_file)

    i = df[variable]

    i = i.to_numpy()

    return sum(len(x) for x in turning_points(i)), turning_points(i)
